// Перед запуском необходимо запустить кэширующую технологию Redis, используя команду ниже:
//redis-server /usr/local/etc/redis.conf
//Разработка программного интерфейса взаимодействия системы отображения метеорологических данных
// с базой данных измерительной информации
//Программный интерфейс на архитектуре REST
//Для получения данных необходимо подключить разработанный программный интерфейс и отправить GET-запрос по адресу:
//название сервера/api/название того route, который возвращает нужные данные
module.exports = router; //экспортируем модуль роутер
// секция подключения библиотек и модулей
const express = require('express'); //Импорт фреймворка экспресс
const redis = require('redis'); //Импорт NO-SQl базы данных Redis
const fs = require('fs'); //Импорт модуля, позволяющего работать с файловой системой
const moment = require('moment'); //Импорт библиотеки для работы со временем
const request = require('request'); //Импорт библиотеки, отправляющей асинхронные запросы
var router = express.Router();

//---Декларация всего неоюходимого для Redis
const port_redis = process.env.PORT || 6379; // Устанавливаем порт
const redis_client = redis.createClient(port_redis); //Создаем подключение по заданному порту
//---Конец блока

// После перезагрузки/повторного запуска очищаем хранилище Redis
redis_client.flushdb(function (err, succeeded) {
    console.log(succeeded);
});


/* Корневой интерфейс, необходимый для тестирования подключения API */
router.get('/', function (req, res) {
    res.send('respond with a resource');
});

//Функция запроса по определенному url
//Возвращает тело ответа на запрос
function downloadPage(url) {
    return new Promise((resolve, reject) => {
        request(url, (error, response, body) => {
            if (error) reject(error);
            if (response.statusCode !== 200) {
                reject(response);
            }
            resolve(body); //Вернуть ответ на запрос
        });
    });
}


// Функция парсинга файла с калибровочными данными
// Возвращает имена приборов и их серийные номера, в случае, если для них есть калибровочные коэффициенты
function get_calibration_name_and_serial(file) {
    return new Promise((resolve, reject) => {
        fs.readFile(file, 'utf-8', function (error, data) { //reading file with json
            if (error) reject(error);
            let obj = JSON.parse(data);
            let array = Object.keys(obj);
            let respond_array = [];
            for (let i = 0; i < array.length; i++) {
                let respond_array_pair = [];
                let device_uname = obj[array[i]].uName;
                let device_serial = obj[array[i]].serial;
                respond_array_pair.push(device_uname);
                respond_array_pair.push(device_serial);
                respond_array.push(respond_array_pair);
            }
            resolve(respond_array);
        });
    });
}

//Функция поиска заданного прибора в списке калибровочных приборов, указав его uname и serial
// В случае ненахождения вернет -1, в противном случае вернет индекс из массива приборов, доступных для калибровки
function device_finder(uname, serial, device_list) {
    return new Promise((resolve, reject) => {
        let index;
        let request_uname_serial = [uname, serial];
        for (let i = 0; i < device_list.length; i++) {
            if (device_list[i][0] === request_uname_serial[0] && device_list[i][1] === request_uname_serial[1]) {
                index = i;
                resolve(index); //Вернуть индекс в массиве приборов, доступных для калибровки
            }
        }
        resolve(-1);
    });
}

// Функция чтения калибровочного файла
// Возвращает содержимое файла
function read_calibration_file(file) {
    return new Promise((resolve, reject) => {
        fs.readFile(file, 'utf-8', function (error, data) { //reading file with json
            if (error) reject(error);
            let obj = JSON.parse(data);
            resolve(obj); //Вернуть содержимое калибровочного файла
        });
    });
}


// Возвращает массив с названиями сенсоров для калибровки
function get_calibration_sensorname(file) {
    return new Promise((resolve, reject) => {
        fs.readFile(file, 'utf-8', function (error, data) { //reading file with json
            if (error) reject(error);
            let obj = JSON.parse(data);
            let array = Object.keys(obj);
            let sensor_array = [];
            let resp_array = [];
            for (let i = 0; i < obj.length; i++) {
                let sensors = obj[i].sensors;
                for (let j = 0; j < sensors.length; j++) {
                    sensor_array.push(sensors[j].sensor);
                }
                resp_array.push(sensor_array);
                sensor_array = [];
            }
            resolve(resp_array);
        });
    });
}



// С таймаутом 600000 делать запрос к базе, на случай если появятся новые калибровочные данные.
// Таким образом будет производиться учет регулярных обновлений калибровочных коэффициентов
var calibration_data = setInterval(function () {
    request({
        url: "http://188.35.161.31/core/deb.php?calibration",
        method: "GET", //Запрос типа GET
        timeout: 600000,
        followRedirect: true,
        maxRedirects: 10
    }, function (error, response, body) {
        if (!error && response.statusCode == 200) {
            console.log('Calibration Data has been loaded');
            fs.writeFile("CalibrationData.txt", response.body.toString(), function (error) {
                if (error) throw error;
            });

        } else {
            console.log('error' + response.statusCode);
        }
    });
}, 600000);

//

// Запись в кеш и лог-файл на сервере для данных за последние 5 минут
var requestLoop_five_min = setInterval(function () {
    //------Блок, отвечающий за формирование времени//
    let now = moment();
    let past = now.subtract(5, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;
//---Конец---//
    request({ //Делаем запрос к базе, получая данные за последние 5 минут
        url: "http://188.35.161.31/core/deb.php?fdate=" + base_requester,
        method: "GET", //Метод запроса - GET
        timeout: 5000, // выставление таймаута
        followRedirect: true,
        maxRedirects: 10  //максимальная попытка переподключений в случае обвала соединения
    }, function (error, response, body) {
        if (!error && response.statusCode == 200) {
            redis_client.set("Rest#Not_calibr#LastLog", response.body.toString(), 'EX', 30); //Запомнить в редис
            fs.writeFile("LastLog.txt", response.body.toString(), function (error) { //записать в лог-файл
                if (error) throw error;
            });

        } else {
            console.log('error' + response.statusCode);
        }
    });
}, 30000);

// Запись в кеш и лог-файл на сервере для данных за последние 7 дней
var requestLoop_weekly = setInterval(function () {
    //Последние 7 дней
    let now_week = moment();
    let real_now_week = moment();
    real_now_week = real_now_week.format("YYYY-MM-DD%20HH:mm:ss");
    let past_seven_days = now_week.clone().subtract(7, 'days');
    past_seven_days = past_seven_days.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_sevendays = past_seven_days + "&sdate=" + real_now_week;
//----
    request({
        url: "http://188.35.161.31/core/deb.php?fdate=" + base_requester_sevendays,
        method: "GET", //запрос типа GET
        timeout: 5000,
        followRedirect: true,
        maxRedirects: 10
    }, function (error, response, body) {
        if (!error && response.statusCode == 200) {
            redis_client.set("Rest#Not_calibr#Week", response.body.toString(), 'EX', 30); //запись в редис
            fs.writeFile("WeekLog.txt", response.body.toString(), function (error) { //запись в лог-файл
                if (error) throw error;
            });

        }
    });
}, 30000);

/*


  _   _           _        _____           _   _   _
 | \ | |         | |      / ____|         | | (_) | |
 |  \| |   ___   | |_    | |        __ _  | |  _  | |__    _ __
 | . ` |  / _ \  | __|   | |       / _` | | | | | | '_ \  | '__|
 | |\  | | (_) | | |_    | |____  | (_| | | | | | | |_) | | |
 |_| \_|  \___/   \__|    \_____|  \__,_| |_| |_| |_.__/  |_|



 */

//router для получения НЕОТКАЛИБРОВАННЫХ данных за последние 5 минут
//Пример URL: http://localhost:3000/api/not_calibr/last5min
router.get('/not_calibr/last5min', function (req, res) {
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(5, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            redis_client.set(redis_key, respond, 'EX', 30); //сохранить данные в редис
            res.send(respond) ? JSON.stringify(respond) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Формируем URL для запроса к БД
    let redis_key = "Rest#Not_calibr#LastLog"; //Формируем ключ, для хранения в Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за последнюю минуту
//Пример URL: http://localhost:3000/api/not_calibr/lastmin
router.get('/not_calibr/lastmin', function (req, res) {
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(1, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            redis_client.set(redis_key, respond, 'EX', 30); //сохранить данные в редис
            res.send(respond) ? JSON.stringify(respond) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Формируем URL для запроса к БД
    let redis_key = "Rest#Not_calibr#LastMin"; //Формируем ключ, для хранения в Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за последние 5 минут с конкретного прибора
//Необходимо указать uname-имя прибора, serial - его серийный номер
//Пример URL: http://localhost:3000/api/not_calibr/last5min/РОСА-П/01
router.get('/not_calibr/last5min/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(5, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "LastLog" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ(за 10 минут) с определенного прибора
//Необходимо указать uname-имя прибора, serial - его серийный номер
//Пример URL: http://localhost:3000/api/not_calibr/last_measurement/РОСА-П/01
router.get('/not_calibr/last_measurement/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[respond_js[array[i]].uName] = value_pair;
                    break;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "LastMeasurement" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


//router для получения НЕОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ(за 10 минут) со всех приборов
//Пример URL: http://localhost:3000/api/not_calibr/last_measurement
router.get('/not_calibr/last_measurement', function (req, res) {
    //------Блок, отвечающий за время к базе - 60 минут----//
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if (respond_array[respond_js[array[i]].uName]===undefined) {
                    let key = respond_js[array[i]].uName;
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[key] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "LastMeasurement"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


// Функция чтения калибровочного файла
// Возвращает содержимое файла
function read_coordinates_file(file) {
    return new Promise((resolve, reject) => {
        fs.readFile(file, 'utf-8', function (error, data) { //reading file with json
            if (error) reject(error);
            data = data.toString()
                .split('\n')
                .map(e => e.trim())
                .map(e => e.split(';').map(e => e.trim()));
            //console.log(data);
            resolve(data); //Вернуть содержимое калибровочного файла
        });
    });
}

//Функция поиска заданного прибора для добавления координат
// В случае ненахождения вернет -1, в противном случае вернет индекс из массива приборов, для которых есть координаты
function coordinate_finder(uname, serial, device_list) {
    return new Promise((resolve, reject) => {
        let index;
        let request_uname_serial = [uname, serial];
        for (let i = 0; i < device_list.length; i++) {
            if (device_list[i][1] === request_uname_serial[0] && device_list[i][2] === request_uname_serial[1]) {
                index = i;
                resolve(index); //Вернуть индекс в массиве приборов, доступных для калибровки
            }
        }
        resolve(-1);
    });
}


//router для получения НЕОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ(за 10 минут) с определенного прибора С КООРДИНАТАМИ
//Необходимо указать uname-имя прибора, serial - его серийный номер
//Пример URL: http://localhost:3000/api/not_calibr/last_measurement/РОСА-П/01
router.get('/not_calibr/last_measurement/coordinates/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let device_list = await read_coordinates_file("Coordinates.csv");
            array = array.reverse();
            let respond_array = {};
            let index = await coordinate_finder(request_uname,request_serial,device_list);
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[respond_js[array[i]].uName] = value_pair;
                    if(index!=-1){
                        respond_array[request_uname].x = device_list[index][3];
                        respond_array[request_uname].y = device_list[index][4];
                    }
                    break;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "LastMeasurement" +"#"+ "Coordinates"+ "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});




//router для получения НЕОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ(за 10 минут) со всех приборов С КООРДИНАТАМИ
//Пример URL: http://localhost:3000/api/not_calibr/last_measurement/coordinates
router.get('/not_calibr/last_measurement/coordinates', function (req, res) {
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let coordinates_file_content = await read_coordinates_file("Coordinates.csv");
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if (respond_array[respond_js[array[i]].uName]===undefined) {
                    let key = respond_js[array[i]].uName;
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[key] = value_pair;
                    let index = await coordinate_finder(respond_array[key].uName, respond_array[key].serial,coordinates_file_content);
                    if (index !=-1){
                        respond_array[key].x = coordinates_file_content[index][3];
                        respond_array[key].y = coordinates_file_content[index][4];
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "LastMeasurement"+"#"+"Coordinates"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


//router для получения НЕОТКАЛИБРОВАННЫХ данных за последнюю минуту с конкретного прибора
//Необходимо указать uname-имя прибора, serial - его серийный номер
//Пример URL: http://localhost:3000/api/not_calibr/lastmin/РОСА-П/01
router.get('/not_calibr/lastmin/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //------Блок, отвечающий за время к базе - последняя минута ----//
    let now = moment();
    let past = now.subtract(1, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#" +"Not_calibr" + "#" + "Lastmin" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за последнию неделю
//Пример URL: http://localhost:3000/api/not_calibr/lastweek
router.get('/not_calibr/lastweek', function (req, res) {
    //Формирование времени за последние 7 дней
    let now_week = moment();
    let real_now_week = moment();
    real_now_week = real_now_week.format("YYYY-MM-DD%20HH:mm:ss");
    let past_seven_days = now_week.clone().subtract(7, 'days');
    past_seven_days = past_seven_days.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_sevendays = past_seven_days + "&sdate=" + real_now_week;

    //---Конец
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url);
            redis_client.set(redis_key, respond, 'EX', 30);
            res.send(respond) ? JSON.stringify(respond) : {};
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_sevendays; //Генерируем URL для запроса к БД
    let redis_key = "Rest#Not_calibr#Week"; //Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});
//router для получения НЕОТКАЛИБРОВАННЫХ данных за последнюю неделю с конкретного прибора
//Необходимо указать uname-имя прибора, serial - его серийный номер
//Пример URL: http://localhost:3000/api/not_calibr/lastweek/РОСА-П/01
router.get('/not_calibr/lastweek/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname; //Имя запрашиваемого прибора
    let request_serial = req.params.serial; //Серийный номер запрашиваемого прибора
    //Формирование времени за последнюю неделю
    let now_week = moment();
    let real_now_week = moment();
    real_now_week = real_now_week.format("YYYY-MM-DD%20HH:mm:ss");
    let past_seven_days = now_week.clone().subtract(7, 'days');
    past_seven_days = past_seven_days.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_sevendays = past_seven_days + "&sdate=" + real_now_week;

    //--Конец
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_sevendays; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Not_calibr" + "#" + "Week" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за день с конкретного прибора
//Необходимо указать uname-имя прибора, serial - его серийный номер
//date - дата конкретного дня, в формате "Год-месяц-день". Пример: 2019-04-04
//Пример URL: http://localhost:3000/api/not_calibr/day/2019-04-04/РОСА-П/01
router.get('/not_calibr/day/:data/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname; //Имя запрашиваемого прибора
    let request_serial = req.params.serial; //Серийный номер запрашиваемого прибора
    //Формирование времени за сутки
    let day = req.params.data;
    let till_next_day = req.params.data;
    let first_date = day;
    let second_date = moment(till_next_day).add(24, 'hours').format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_day = first_date + "&sdate=" + second_date;

    //---Конец
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_day; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Not_calibr" + "#" + day + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за день
//date - дата конкретного дня, в формате "Год-месяц-день". Пример: 2019-04-04
//Пример URL: http://localhost:3000/api/not_calibr/day/2019-04-04
router.get('/not_calibr/day/:data', function (req, res) {
    //Блок формирования времени за определенные сутки
    let day = req.params.data;
    let till_next_day = req.params.data;
    let first_date = day;
    let second_date = moment(till_next_day).add(24, 'hours').format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_day = first_date + "&sdate=" + second_date;

    //---Конец
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url);
            redis_client.set(redis_key, respond, 'EX', 30);
            res.send(respond) ? JSON.stringify(respond) : {};
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_day;//Генерируем URL для запроса к базе
    let redis_key ="Rest"+"#"+ "Not_calibr" + "#" + day; //Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


//router для получения НЕОТКАЛИБРОВАННЫХ данных за указанный временной интервал
//Необходимо указать fdate-начало временного интервала, sdate - конец временного интервала
//Формат fdate и sdate: год-месяц-день%20часы:минуты:секунды. Пример: 2019-04-04%2000:00:00
//Пример URL: http://localhost:3000/api/not_calibr/log/2019-04-04%2000:00:00/2019-04-04%2010:10:10
router.get('/not_calibr/log/:fdate/:sdate', function (req, res) {

// функция ответа на запрос клиента
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url);
            redis_client.set(redis_key, respond, 'EX', 30);
            res.send(respond) ? JSON.stringify(respond) : {};
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let start_date = req.params.fdate; //начальная дата запрошенного интервала
    let finish_date = req.params.sdate;//конечная дата запрошенного интервала
    let url = "http://188.35.161.31/core/deb.php?fdate=" + start_date + "&sdate=" + finish_date;//Генерируем URL для бд
    let redis_key ="Rest"+"#"+ "Not_calibr" + "#" + start_date + "#" + finish_date; //Генерируем ключ redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) { // Если данные есть в redis, то отдать из кеша
            return res.json(JSON.parse(data));
        } else { //иначе сделать запрос к базе
            myreq(url, redis_key);
        }
    });
});

//router для получения НЕОТКАЛИБРОВАННЫХ данных за указанный временной интервал с конкретного прибора
//Необходимо указать fdate-начало временного интервала, sdate - конец временного интервала
//Формат fdate и sdate: год-месяц-день%20часы:минуты:секунды. Пример: 2019-04-04%2000:00:00
//Также необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/not_calibr/log/2019-04-04%2000:00:00/2019-04-04%2010:10:10/РОСА-П/01
router.get('/not_calibr/log/:fdate/:sdate/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname; //Имя запрошенного прибора
    let request_serial = req.params.serial; //Серийный номер запрошенного прибора

    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let start_date = req.params.fdate; //get start_date from request
    let finish_date = req.params.sdate;//get finish_date frim request
    let url = "http://188.35.161.31/core/deb.php?fdate=" + start_date + "&sdate=" + finish_date;//Генерируем URL для бд
    let redis_key ="Rest" +"#"+"Not_calibr" + "#" + start_date + "#" + finish_date + "#" + request_uname + "#" + request_serial;//Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) { // Если данные есть в redis, то отдать из кеша
            return res.json(JSON.parse(data));
        } else { //иначе сделать запрос к базе
            myreq(url, redis_key);
        }
    });
});
/*
   _____           _   _   _
  / ____|         | | (_) | |
 | |        __ _  | |  _  | |__    _ __
 | |       / _` | | | | | | '_ \  | '__|
 | |____  | (_| | | | | | | |_) | | |
  \_____|  \__,_| |_| |_| |_.__/  |_|
 */

//router для получения ОТКАЛИБРОВАННЫХ данных за указанный временной интервал
//Необходимо указать fdate-начало временного интервала, sdate - конец временного интервала
//Формат fdate и sdate: год-месяц-день%20часы:минуты:секунды. Пример: 2019-04-04%2000:00:00
//Пример URL: http://localhost:3000/api/calibr/log/2019-04-04%2000:00:00/2019-04-04%2010:10:10/РОСА-П/01
router.get('/calibr/log/:fdate/:sdate', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_js[array[i]].uName, respond_js[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_js[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_js), 'EX', 30); //сохранить данные в редис
            res.send(respond_js) ? JSON.stringify(respond_js) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let start_date = req.params.fdate; //начальная дата интервала
    let finish_date = req.params.sdate;//Конечная дата интервала
    let url = "http://188.35.161.31/core/deb.php?fdate=" + start_date + "&sdate=" + finish_date;//Генерируем URL для бд
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + start_date + "#" + finish_date;//Генерируем ключ Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) { // Если данные есть в redis, то отдать из кеша
            return res.json(JSON.parse(data));
        } else { //иначе сделать запрос к базе
            myreq(url, redis_key);
        }
    });
});

//router для получения ОТКАЛИБРОВАННЫХ данных за последние 5 минут
//Пример URL: http://localhost:3000/api/calibr/last5min
router.get('/calibr/last5min', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_js[array[i]].uName, respond_js[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_js[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_js), 'EX', 30); //сохранить данные в редис
            res.send(respond_js) ? JSON.stringify(respond_js) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(5, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;
    //---Конец
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastLog"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});

//router для получения ОТКАЛИБРОВАННЫХ данных за последнюю минуту
//Пример URL: http://localhost:3000/api/calibr/lastmin
router.get('/calibr/lastmin', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_js[array[i]].uName, respond_js[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_js[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_js), 'EX', 30); //сохранить данные в редис
            res.send(respond_js) ? JSON.stringify(respond_js) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(1, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;
    //---Конец
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMin"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});

//router для получения ОТКАЛИБРОВАННЫХ данных за последнюю минуту с конкретного прибора
//Необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/lastmin/РОСА-П/01
router.get('/calibr/lastmin/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(1, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//
// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMin" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


//router для получения ПОСЛЕДНЕГО ОТКАЛИБРОВАННОГО ИЗМЕРЕНИЯ С КАЖДОГО ПРИБОРА, КОТОРЫЙ ПРОИЗВОДИЛ ИЗМЕРЕНИЯ ЗА ПОСЛЕДНИЙ ЧАС
//Пример URL: http://localhost:3000/api/calibr/last_measurement
router.get('/calibr/last_measurement', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if (respond_array[respond_js[array[i]].uName]===undefined) {
                    let key = respond_js[array[i]].uName;
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[key] = value_pair;
                }
            }
            array = Object.keys(respond_array);
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_array[array[i]].uName, respond_array[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;
    //---Конец
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMeasurement"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});



//router для получения ПОСЛЕДНЕГО ОТКАЛИБРОВАННОГО ИЗМЕРЕНИЯ С КАЖДОГО ПРИБОРА С КООРДИНАТАМИ
//Пример URL: http://localhost:3000/api/calibr/last_measurement/coordinates
router.get('/calibr/last_measurement/coordinates', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let coordinates_file_content = await read_coordinates_file("Coordinates.csv");
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if (respond_array[respond_js[array[i]].uName]===undefined) {
                    let key = respond_js[array[i]].uName;
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[key] = value_pair;
                    let index = await coordinate_finder(respond_array[key].uName, respond_array[key].serial,coordinates_file_content);
                    if (index !=-1){
                        respond_array[key].x = coordinates_file_content[index][3];
                        respond_array[key].y = coordinates_file_content[index][4];
                    }
                }
            }
            array = Object.keys(respond_array);
            for (let i = 0; i < array.length; i++) {
                let index = await device_finder(respond_array[array[i]].uName, respond_array[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;
    //---Конец
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMeasurement"+"#"+"Coordinates"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});


//router для получения ОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ ДЛЯ КОНКРЕТНОГО ПРИБОРА C КООРДИНАТАМИ
//Необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/last_measurement/coordinates/РОСА-П/01
router.get('/calibr/last_measurement/coordinates/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;

    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;


// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let device_list = await read_coordinates_file("Coordinates.csv");
            let index_coord = await coordinate_finder(request_uname,request_serial,device_list);
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            array = array.reverse();
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    if (respond_array[respond_js[array[i]].uName]===undefined) {
                        let key = respond_js[array[i]].uName;
                        let key_pair = array[i];
                        let value_pair = respond_js[key_pair];
                        respond_array[key] = value_pair;
                        if(index_coord!=-1){
                            respond_array[respond_js[array[i]].uName].x = device_list[index_coord][3];
                            respond_array[respond_js[array[i]].uName].y = device_list[index_coord][4];
                        }
                    }
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMeasurement" +"#"+ "Coordinates"+ "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


//router для получения ОТКАЛИБРОВАННОГО ПОСЛЕДНЕГО ИЗМЕРЕНИЯ ДЛЯ КОНКРЕТНОГО ПРИБОРА ЗА ПОСЛЕДНИЙ ЧАС
//Необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/last_measurement/РОСА-П/01
router.get('/calibr/last_measurement/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//
// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            array = array.reverse();
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    if (respond_array[respond_js[array[i]].uName]===undefined) {
                        let key = respond_js[array[i]].uName;
                        let key_pair = array[i];
                        let value_pair = respond_js[key_pair];
                        respond_array[key] = value_pair;
                    }
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastMeasurement" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});



//router для получения ОТКАЛИБРОВАННЫХ данных за последние 5 минут конкретного прибора
//Необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/last5min/РОСА-П/01
router.get('/calibr/last5min/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //------Блок, отвечающий за время к базе - 5 минут----//
    let now = moment();
    let past = now.subtract(5, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

//---Конец---//
// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "LastLog" + "#" + request_uname + "#" + request_serial; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});
//router для получения ОТКАЛИБРОВАННЫХ данных за указанный временной интервал с конкретного прибора
//Необходимо указать fdate-начало временного интервала, sdate - конец временного интервала
//Формат fdate и sdate: год-месяц-день%20часы:минуты:секунды. Пример: 2019-04-04%2000:00:00
//Также необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/log/2019-04-04%2000:00:00/2019-04-04%2010:10:10/РОСА-П/01
router.get('/calibr/log/:fdate/:sdate/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;

    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let start_date = req.params.fdate; //get start_date from request
    let finish_date = req.params.sdate;//get finish_date frim request
    let url = "http://188.35.161.31/core/deb.php?fdate=" + start_date + "&sdate=" + finish_date;//Генерируем URL для БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + start_date + "#" + finish_date + "#" + request_uname + "#" + request_serial;//Генерируем ключ для Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) { // Если данные есть в redis, то отдать из кеша
            return res.json(JSON.parse(data));
        } else { //иначе сделать запрос к базе
            myreq(url, redis_key);
        }
    });
});
//router для получения ОТКАЛИБРОВАННЫХ данных за последнюю неделю
//Пример URL: http://localhost:3000/api/calibr/lastweek
router.get('/calibr/lastweek', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_js[array[i]].uName, respond_js[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_js[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_js), 'EX', 30); //сохранить данные в редис
            res.send(respond_js) ? JSON.stringify(respond_js) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //Генерация даты за последнюю неделю
    let now_week = moment();
    let real_now_week = moment();
    real_now_week = real_now_week.format("YYYY-MM-DD%20HH:mm:ss");
    let past_seven_days = now_week.clone().subtract(7, 'days');
    past_seven_days = past_seven_days.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_sevendays = past_seven_days + "&sdate=" + real_now_week;
    //---Конец
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_sevendays; //Генерация url для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "Week"; //Генерация ключа Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});

//router для получения ОТКАЛИБРОВАННЫХ данных за последнюю неделю с конкретного прибора
//Необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/lastweek/РОСА-П/01
router.get('/calibr/lastweek/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    //Генерация даты за последнюю неделю
    let now_week = moment();
    let real_now_week = moment();
    real_now_week = real_now_week.format("YYYY-MM-DD%20HH:mm:ss");
    let past_seven_days = now_week.clone().subtract(7, 'days');
    past_seven_days = past_seven_days.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_sevendays = past_seven_days + "&sdate=" + real_now_week;

    //---Конец

    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_sevendays; //Генерация URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + "Week" + "#" + request_uname + "#" + request_serial; //Генерация ключа Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});
//router для получения ОТКАЛИБРОВАННЫХ данных за определенный день с конкретного прибора
//Необходимо указать date - конкретную дату дня в формате "Год - месяц - день". Пример:2019-04-04
//Также необходимо указать название и серийный номер прибора
//Пример URL: http://localhost:3000/api/calibr/day/2019-04-04/РОСА-П/01
router.get('/calibr/day/:data/:uname/:serial', function (req, res) {
    let request_uname = req.params.uname;
    let request_serial = req.params.serial;
    let day = req.params.data;
    let till_next_day = req.params.data;
    let first_date = day;
    let second_date = moment(till_next_day).add(24, 'hours').format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_day = first_date + "&sdate=" + second_date;

    // Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            let index = await device_finder(request_uname, request_serial, available_devices);
            let respond_array = {};
            for (let i = 0; i < array.length; i++) {
                if ((respond_js[array[i]].uName === request_uname) && (respond_js[array[i]].serial === request_serial)) {
                    var key_pair = array[i];
                    var value_pair = respond_js[key_pair];
                    respond_array[key_pair] = value_pair;
                }
            }
            if (index !== -1) {
                array = Object.keys(respond_array);
                for (let i = 0; i < array.length; i++) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_array[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            redis_client.set(redis_key, JSON.stringify(respond_array), 'EX', 30); //сохранить данные в редис
            res.send(respond_array) ? JSON.stringify(respond_array) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }


    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_day; //Генерация URL для запроса к БД
    let redis_key ="Rest"+"#"+"Calibr" + "#" + day + "#" + request_uname + "#" + request_serial; //Генерация ключа Redis
    return redis_client.get(day, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});
//router для получения ОТКАЛИБРОВАННЫХ данных за определенный день
//Необходимо указать date - дату дня в формате "Год - месяц - день". Пример: 2019-04-04
//Пример URL: http://localhost:3000/api/calibr/day/2019-04-04
router.get('/calibr/day/:data', function (req, res) {
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            const respond_sensor_name = await get_calibration_sensorname('CalibrationData.txt');
            const calibration_file_content = await read_calibration_file('CalibrationData.txt');
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            const available_devices = await get_calibration_name_and_serial('CalibrationData.txt');
            for (let i = 0; i < array.length; i++) {
                var index = await device_finder(respond_js[array[i]].uName, respond_js[array[i]].serial, available_devices);
                if (index !== -1) {
                    let date_array = respond_sensor_name[index];
                    for (let k = 0; k < date_array.length; k++) {
                        let sensor_name = date_array[k];
                        let sensor_index = Object.keys(respond_js[array[i]].data).indexOf(sensor_name);
                        let calibr_datetime;
                        let measurement_datetime = moment(new Date(respond_js[array[i]].Date));
                        for(let x = 0; x <calibration_file_content[index].sensors[k].calibr.length;x++)
                        {
                            calibr_datetime = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].datetime);
                            calibr_datetime = moment(new Date(calibr_datetime));
                            if (measurement_datetime.isAfter(calibr_datetime) === false){
                                let calibr_n = JSON.stringify(calibration_file_content[index].sensors[k].calibr[x].data.n);
                                if (calibr_n == 1) {
                                    calibr_ai = calibration_file_content[index].sensors[k].calibr[x].data.ai[0];
                                    if (calibr_ai !== undefined) {
                                        let obj = respond_js[array[i]].data;
                                        let arr = Object.entries(obj);
                                        if (arr[sensor_index] !== undefined) {
                                            arr[sensor_index][1] = (parseFloat(arr[sensor_index][1]) + calibr_ai).toFixed(2);
                                        }
                                        respond_js[array[i]].data = Object.fromEntries(arr);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_js), 'EX', 30); //сохранить данные в редис
            res.send(respond_js) ? JSON.stringify(respond_js) : {};//отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let day = req.params.data;
    let till_next_day = req.params.data;
    let first_date = day;
    let second_date = moment(till_next_day).add(24, 'hours').format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester_day = first_date + "&sdate=" + second_date;
    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester_day;//Генерация URL для запроса к БД
    let redis_key ="Rest"+"#"+ "Calibr" + "#" + day; //Генерация ключа Redis
    return redis_client.get(redis_key, (err, data) => {
        if (data) {
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });

});
/*
   _____   _                   __  __
  / ____| (_)                 |  \/  |
 | |  __   _   _ __           | \  / |   __ _   _ __    ___
 | | |_ | | | | '__|          | |\/| |  / _` | | '_ \  / __|
 | |__| | | | | |             | |  | | | (_| | | |_) | \__ \
  \_____| |_| |_|             |_|  |_|  \__,_| | .__/  |___/
                      ______                   | |
                     |______|                  |_|
 */


//router для получения координат, с которых были последние измерения(ДЛЯ ВСЕХ ПРИБОРОВ)
//Пример URL: http://localhost:3000/api/maps/last_measurement/coordinates
router.get('/maps/last_measurement/coordinates', function (req, res) {
    let now = moment();
    let past = now.subtract(10, 'minutes');
    past = past.format("YYYY-MM-DD%20HH:mm:ss");
    let real_now = moment();
    real_now = real_now.format("YYYY-MM-DD%20HH:mm:ss");
    let base_requester = past + "&sdate=" + real_now;

// Функция для ответа клиенту, который запросил данные
    async function myreq(url, redis_key) {
        try {
            const respond = await downloadPage(url); //сделать запрос к базе
            let coordinates_file_content = await read_coordinates_file("Coordinates.csv");
            let respond_js = JSON.parse(respond);
            let array = Object.keys(respond_js);
            array = array.reverse();
            let respond_array = {};
            let respond_array_coordinates = [];
            //Можно конечно было не формировать список измерений попутно, но, т.к. это вероятно придется сделать
            //То пусть будет, много оперативы не ест
            for (let i = 0; i < array.length; i++) {
                if (respond_array[respond_js[array[i]].uName]===undefined) {
                    let key = respond_js[array[i]].uName;
                    let key_pair = array[i];
                    let value_pair = respond_js[key_pair];
                    respond_array[key] = value_pair;
                    let index = await coordinate_finder(respond_array[key].uName, respond_array[key].serial,coordinates_file_content);
                    if (index !=-1){
                        respond_array[key].x = coordinates_file_content[index][3];
                        respond_array[key].y = coordinates_file_content[index][4];
                        respond_array_coordinates.push([respond_array[key].x,respond_array[key].y]);
                    }
                }
            }
            redis_client.set(redis_key, JSON.stringify(respond_array_coordinates), 'EX', 30); //сохранить данные в редис
            res.send(respond_array_coordinates) ? JSON.stringify(respond_array_coordinates) : {}; //отправить клиенту ответ на его запрос
        } catch (error) {
            console.error('ERROR:');
            console.error(error);
        }
    }

    let url = "http://188.35.161.31/core/deb.php?fdate=" + base_requester; //Генерируем URL для запроса к БД
    let redis_key ="Maps"+"#"+"LastMeasurement"+"#"+"Coordinates"; //Генерируем ключ Redis
    // Сначала идет поиск данных в редис. Если они есть - отдать из кеша, иначе - сделать запрос к базе.
    return redis_client.get(redis_key, (err, data) => {
        if (data) { //если в редисе есть данные
            console.log(data);
            return res.json(JSON.parse(data));
        } else {
            myreq(url, redis_key);
        }
    });
});


module.exports = router;


